local L = LibStub("AceLocale-3.0"):GetLocale("GarrisonJukeBox")

-- Lua APIs
local tinsert, tconcat, tremove, tsort = table.insert, table.concat, table.remove, table.sort
local fmt, tostring = string.format, tostring
local select, pairs, next, type, unpack = select, pairs, next, type, unpack
local loadstring, assert, error = loadstring, assert, error
local setmetatable, getmetatable, rawset, rawget = setmetatable, getmetatable, rawset, rawget
local mrand = math.random

-------------------------------------
-- Extracts the filename from a path+filename string
-------------------------------------
function GJB:ExtractMP3Filename(str)
	local fn = ""
	local last = 1
	local i = 1

	while i ~= nil do
		i = strfind( str, "\\", i )
		if i ~= nil then
			i = i + 1
			last = i
		end
	end

	if last > 1 then
		fn = strsub( str, last )
	end
	
	return fn
end

-- -------------------------------------------
-- function to find out if music file is a PlaySoundKitID callable item
-- -------------------------------------------
function GJB:IsKitMusic(sample)
	if sample[GJB.MT_KITACTIVE] ~= nil and sample[GJB.MT_KITACTIVE] == true then
		return true
	else
		return false
	end
end

-- -------------------------------------------
-- function to sort a musictable
-- -------------------------------------------
function GJB:SortMusicTable(tbl)
	for _, v in pairs(tbl) do
		tsort(v, function(a,b) return a["name"] < b["name"] end)
	end
	return tbl
end

-- -------------------------------------------
-- function to sort a zonetable
-- -------------------------------------------
function GJB:SortZoneTable(tbl)
	for _, v in pairs(tbl) do
		tsort(v, function(a,b) return a[1] < b[1] end)
	end
	return tbl
end

-- -------------------------------------------
-- function to reset profile save if older version detected (1.x)
-- -------------------------------------------
function GJB:VersionCheckResetData()
	GJB:SetMessageBoxText(L["ADDON_BETA_WARNING"])
	GJB.mbox.fbg:Hide()

	-- reset old save
	if self.db.profile.firstrun2003 == nil or self.db.profile.firstrun2003 == true then
		GJB.mbox.fbg:Show()
		local p = self.db.profile
		p.firstrun2003 = false
		p.musiclist = {}
		p.zonelist = {}
		p.songlist = nil
		p.shuffle = nil
		p.inlinetitle = nil
		p.musicplayer.dim = nil
		p.advzonelist = {}
		p.stm_files = {}
		p.stminfo = nil
		p.advenabled = false
		p.stmode = false
		p.allzones = false
	end	
end

-- -------------------------------------------
-- function to store musictable id's as keys to increase searching speed.
-- -------------------------------------------
function GJB:CreateMTID(mt)
	local x = {}
	
	if mt ~= nil then
		-- loop through expansions
		for k, v in ipairs(mt) do
			local t = {}
			-- loop through titles
			for k2, v2 in ipairs(v) do
				local m = {}
				-- loop through music
				for k3, v3 in ipairs(v2.files) do
					m[v3[1]] = k3
				end
				t[v2.id] = m
				t[v2.id].key = k2
			end
			x[k] = t
		end
	end
	
	return x
end

-- -------------------------------------------
-- Table Invert for query of key
-- -------------------------------------------
function GJB:table_invert(t)
   local it={}
   for k,v in pairs(t) do
     it[v]=k
   end
   return it
end
